/*
 * Lilith - a log event viewer.
 * Copyright (C) 2007-2008 Joern Huxhorn
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.huxhorn.lilith.swing;

import de.huxhorn.lilith.data.access.AccessEvent;
import de.huxhorn.lilith.data.logging.LoggingEvent;
import de.huxhorn.lilith.data.logging.ThrowableInfo;
import de.huxhorn.lilith.engine.EventSource;
import de.huxhorn.lilith.data.eventsource.SourceIdentifier;
import de.huxhorn.lilith.data.eventsource.EventWrapper;
import de.huxhorn.lilith.services.sender.EventSender;
import de.huxhorn.sulky.swing.KeyStrokes;
import de.huxhorn.sulky.conditions.Condition;
import org.simplericity.macify.eawt.Application;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.JPopupMenu;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.Map;
import java.util.SortedMap;
import java.io.Serializable;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * This class needs cleanup...... remove duplicated logic, make ToolBar/Menu configurable...
 */
public class ViewActions
{
	private final Logger logger = LoggerFactory.getLogger(ViewActions.class);

	private static final Icon EMPTY_16_ICON;

	private static final Icon CLEAR_MENU_ICON;
	private static final Icon ATTACH_MENU_ICON;
	private static final Icon DETACH_MENU_ICON;
	private static final Icon PAUSED_MENU_ICON;
	private static final Icon UNPAUSED_MENU_ICON;
	private static final Icon FIND_MENU_ITEM;
	private static final Icon STATISTICS_MENU_ICON;
	private static final Icon DISCONNECT_MENU_ICON;
	private static final Icon TAIL_MENU_ICON;
	private static final Icon FIND_NEXT_MENU_ICON;
	private static final Icon FIND_PREV_MENU_ICON;
	private static final Icon HELP_MENU_ICON;
	private static final Icon OPEN_INACTIVE_MENU_ICON;
	private static final Icon EXIT_MENU_ICON;
	private static final Icon PREFERENCES_MENU_ICON;

	private static final Icon CLEAR_TOOLBAR_ICON;
	private static final Icon ATTACH_TOOLBAR_ICON;
	private static final Icon DETACH_TOOLBAR_ICON;
	private static final Icon PAUSED_TOOLBAR_ICON;
	private static final Icon UNPAUSED_TOOLBAR_ICON;
	private static final Icon FIND_TOOLBAR_ICON;
	private static final Icon STATISTICS_TOOLBAR_ICON;
	private static final Icon DISCONNECT_TOOLBAR_ICON;
	private static final Icon TAIL_TOOLBAR_ICON;

	private static final Icon PREFERENCES_TOOLBAR_ICON;

	// TODO: Resource classes
	// TODO: Separate Toolbar and Menu creation.
	// TODO: toolbar action, menu action => same action
	static
	{
		Icon icon;
		{
			URL url=EventWrapperViewPanel.class.getResource("/otherGraphics/empty16.png");
			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		EMPTY_16_ICON =icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/edit-clear.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		CLEAR_MENU_ICON =icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/edit-clear.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		CLEAR_TOOLBAR_ICON =icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/edit-undo.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		ATTACH_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/edit-redo.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		DETACH_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/edit-undo.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		ATTACH_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/edit-redo.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		DETACH_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/media-playback-start.png");
			if(url!=null)
			{
				icon=new ImageIcon(url);
			}
			else
			{
				icon=null;
			}
		}
		PAUSED_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/media-playback-pause.png");
			if(url!=null)
			{
				icon=new ImageIcon(url);
			}
			else
			{
				icon=null;
			}
		}
		UNPAUSED_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/media-playback-start.png");
			if(url!=null)
			{
				icon=new ImageIcon(url);
			}
			else
			{
				icon=null;
			}
		}
		PAUSED_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/media-playback-pause.png");
			if(url!=null)
			{
				icon=new ImageIcon(url);
			}
			else
			{
				icon=null;
			}
		}
		UNPAUSED_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/edit-find.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		FIND_MENU_ITEM=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/edit-find.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		FIND_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/apps/utilities-system-monitor.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		STATISTICS_MENU_ICON =icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/apps/utilities-system-monitor.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		STATISTICS_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/media-eject.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		DISCONNECT_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/media-eject.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		DISCONNECT_TOOLBAR_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/go-down.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		FIND_NEXT_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/go-up.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}
		}
		FIND_PREV_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/go-bottom.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}

		}
		TAIL_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/actions/go-bottom.png");
			if(url!=null)
			{
				icon =new ImageIcon(url);
			}
			else
			{
				icon =null;
			}

		}
		TAIL_TOOLBAR_ICON=icon;

		{
			URL url=MainFrame.class.getResource("/tango/16x16/apps/help-browser.png");

			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		HELP_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/document-open.png");
			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		OPEN_INACTIVE_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/actions/system-log-out.png");
			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		EXIT_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/16x16/categories/preferences-system.png");
			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		PREFERENCES_MENU_ICON=icon;

		{
			URL url=EventWrapperViewPanel.class.getResource("/tango/32x32/categories/preferences-system.png");
			if(url!=null)
			{
				icon = new ImageIcon(url);
			}
			else
			{
				icon = null;
			}
		}
		PREFERENCES_TOOLBAR_ICON=icon;
	}
	private JToolBar toolbar;
	private JMenuBar menubar;

	private MainFrame mainFrame;
	private ViewContainer viewContainer;
	private JToggleButton scrollToBottomButton;
	private AttachToolBarAction attachToolBarAction;
	private AttachMenuAction attachMenuAction;
	private DisconnectToolBarAction disconnectToolBarAction;
	private DisconnectMenuAction disconnectMenuAction;
	private PauseToolBarAction pauseToolBarAction;
	private PauseMenuAction pauseMenuAction;
	private FindPreviousAction findPreviousAction;
	private FindNextAction findNextAction;
	private ResetFindAction resetFindAction;
	private ScrollToBottomMenuAction scrollToBottomMenuAction;
	private EditSourceNameMenuAction editSourceNameMenuAction;
	private EditConditionMenuAction editConditionMenuAction;

	private NextTabAction nextTabAction;
	private PreviousTabAction previousTabAction;
	private CloseFilterAction closeFilterAction;
	private CloseOtherFiltersAction closeOtherFiltersAction;
	private CloseAllFiltersAction closeAllFiltersAction;

	//private ClearAndRemoveInactiveAction clearAndRemoveInactiveAction;
	private RemoveInactiveAction removeInactiveAction;
	private CloseAllAction closeAllAction;
	private CloseAllOtherAction closeAllOtherAction;
	private MinimizeAllAction minimizeAllAction;
	private MinimizeAllOtherAction minimizeAllOtherAction;

	private JMenuItem removeInactiveItem;
	//private JMenuItem clearAndRemoveInactiveItem;

	private JMenu windowMenu;
	private AboutAction aboutAction;
	private PreferencesMenuAction preferencesMenuAction;
	private FindMenuAction findMenuAction;
	private JMenu searchMenu;
	private JMenu viewMenu;
	private ClearMenuAction clearMenuAction;
	private FocusMessageAction focusMessageAction;
	private FocusEventsAction focusEventsAction;
	private StatisticsMenuAction statisticsMenuAction;
	private ChangeListener containerChangeListener;
	private ScrollToBottomToolBarAction scrollToBottomToolBarAction;
	private ClearToolBarAction clearToolBarAction;
	private FindToolBarAction findToolBarAction;
	private StatisticsToolBarAction statisticsToolBarAction;
	private CopyEventAction copyEventAction;
	private CopyLoggingMessageAction copyLoggingMessageAction;
	private CopyLoggingThrowableAction copyLoggingThrowableAction;
	private CopyLoggerNameAction copyLoggerNameAction;
	private CopyAccessUriAction copyAccessUriAction;
	private ShowUnfilteredEventAction showUnfilteredEventAction;
	private JPopupMenu popup;
	private GotoSourceAction gotoSourceAction;
	private JMenu sendToMenuItem;
	private PropertyChangeListener containerPropertyChangeListener;
	private EventWrapper eventWrapper;
	private JMenuItem closeAllItem;
	private JMenuItem minimizeAllItem;
	private JMenuItem closeAllOtherItem;
	private JMenuItem minimizeAllOtherItem;


	public ViewActions(MainFrame mainFrame, ViewContainer viewContainer)
	{
		this(mainFrame);
		setViewContainer(viewContainer);
	}

	public ViewActions(MainFrame mainFrame)
	{
		this.mainFrame=mainFrame;

		containerChangeListener=new ChangeListener()
		{
			/**
			 * Invoked when the target of the listener has changed its state.
			 *
			 * @param e a ChangeEvent object
			 */
			public void stateChanged(ChangeEvent e)
			{
				updateActions();
			}
		};

		containerPropertyChangeListener=new PropertyChangeListener()
		{

			/**
			 * This method gets called when a bound property is changed.
			 *
			 * @param evt A PropertyChangeEvent object describing the event source
			 *            and the property that has changed.
			 */

			public void propertyChange(PropertyChangeEvent evt)
			{
				if(ViewContainer.SELECTED_EVENT_PROPERTY_NAME.equals(evt.getPropertyName()))
				{
					setEventWrapper((EventWrapper) evt.getNewValue());
				}

			}
		};
		// ##### Menu Actions #####
		// File
		OpenInactiveLogMenuAction openInactiveLogMenuAction = new OpenInactiveLogMenuAction();
		CleanAllInactiveLogsMenuAction cleanAllInactiveLogsMenuAction = new CleanAllInactiveLogsMenuAction();
		preferencesMenuAction = new PreferencesMenuAction();
		ExitMenuAction exitMenuAction = new ExitMenuAction();

		// Edit
		showUnfilteredEventAction=new ShowUnfilteredEventAction();
		gotoSourceAction=new GotoSourceAction();
		copyEventAction=new CopyEventAction();
		copyLoggingMessageAction=new CopyLoggingMessageAction();
		copyLoggerNameAction=new CopyLoggerNameAction();
		copyLoggingThrowableAction=new CopyLoggingThrowableAction();
		copyAccessUriAction =new CopyAccessUriAction();


		// Search
		findMenuAction = new FindMenuAction();
		findPreviousAction=new FindPreviousAction();
		findNextAction=new FindNextAction();
		resetFindAction=new ResetFindAction();

		// View
		scrollToBottomMenuAction = new ScrollToBottomMenuAction();
		pauseMenuAction = new PauseMenuAction();
		clearMenuAction = new ClearMenuAction();
		attachMenuAction = new AttachMenuAction();
		disconnectMenuAction = new DisconnectMenuAction();

		focusMessageAction = new FocusMessageAction();
		focusEventsAction = new FocusEventsAction();

		statisticsMenuAction = new StatisticsMenuAction();
		editSourceNameMenuAction = new EditSourceNameMenuAction();
		editConditionMenuAction = new EditConditionMenuAction();

		previousTabAction = new PreviousTabAction();
		nextTabAction = new NextTabAction();
		closeFilterAction = new CloseFilterAction();
		closeOtherFiltersAction = new CloseOtherFiltersAction();
		closeAllFiltersAction = new CloseAllFiltersAction();

		// Window
		closeAllAction=new CloseAllAction();
		closeAllOtherAction=new CloseAllOtherAction();
		minimizeAllAction=new MinimizeAllAction();
		minimizeAllOtherAction=new MinimizeAllOtherAction();
		removeInactiveAction=new RemoveInactiveAction();
		//clearAndRemoveInactiveAction=new ClearAndRemoveInactiveAction();

		// Help
		KeyboardHelpAction keyboardHelpAction = new KeyboardHelpAction();
		DebugAction debugAction=new DebugAction();
		aboutAction = new AboutAction();
		CheckForUpdateAction checkForUpdateAction = new CheckForUpdateAction();

		// ##### ToolBar Actions #####
		scrollToBottomToolBarAction = new ScrollToBottomToolBarAction();
		pauseToolBarAction = new PauseToolBarAction();
		clearToolBarAction = new ClearToolBarAction();
		findToolBarAction = new FindToolBarAction();
		statisticsToolBarAction = new StatisticsToolBarAction();
		attachToolBarAction = new AttachToolBarAction();
		PreferencesToolBarAction preferencesToolBarAction = new PreferencesToolBarAction();
		disconnectToolBarAction = new DisconnectToolBarAction();

		closeAllItem=new JMenuItem(closeAllAction);
		closeAllOtherItem=new JMenuItem(closeAllOtherAction);
		minimizeAllItem=new JMenuItem(minimizeAllAction);
		minimizeAllOtherItem=new JMenuItem(minimizeAllOtherAction);
		removeInactiveItem = new JMenuItem(removeInactiveAction);
		//clearAndRemoveInactiveItem = new JMenuItem(clearAndRemoveInactiveAction);

		toolbar=new JToolBar(SwingConstants.HORIZONTAL);
		toolbar.setFloatable(false);


		scrollToBottomButton=new JToggleButton(scrollToBottomToolBarAction);
		toolbar.add(scrollToBottomButton);

		JButton pauseButton = new JButton(pauseToolBarAction);
		toolbar.add(pauseButton);

		JButton clearButton = new JButton(clearToolBarAction);
		toolbar.add(clearButton);

		JButton findButton = new JButton(findToolBarAction);
		toolbar.add(findButton);

		JButton disconnectButton = new JButton(disconnectToolBarAction);
		toolbar.add(disconnectButton);

		toolbar.addSeparator();

		JButton statisticsButton = new JButton(statisticsToolBarAction);
		toolbar.add(statisticsButton);

		toolbar.addSeparator();

		JButton attachButton = new JButton(attachToolBarAction);
		toolbar.add(attachButton);

		toolbar.addSeparator();

		JButton preferencesButton = new JButton(preferencesToolBarAction);
		toolbar.add(preferencesButton);

		Application app=mainFrame.getApplication();

		menubar=new JMenuBar();

		// File
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('f');
		fileMenu.add(new JMenuItem(openInactiveLogMenuAction));
		fileMenu.add(new JMenuItem(cleanAllInactiveLogsMenuAction));
		if(!app.isMac())
		{
			fileMenu.addSeparator();
			fileMenu.add(new JMenuItem(preferencesMenuAction));
			fileMenu.addSeparator();
			fileMenu.add(new JMenuItem(exitMenuAction));
		}

		// Edit
		JMenu editMenu = new JMenu("Edit");
		editMenu.setMnemonic('e');
		editMenu.add(copyEventAction);
		editMenu.addSeparator();
		editMenu.add(copyLoggingMessageAction);
		editMenu.add(copyLoggerNameAction);
		editMenu.add(copyLoggingThrowableAction);
		editMenu.addSeparator();
		editMenu.add(copyAccessUriAction);

		// Search
		searchMenu = new JMenu("Search");
		searchMenu.setMnemonic('s');
		searchMenu.add(findMenuAction);
		searchMenu.add(resetFindAction);
		searchMenu.add(findPreviousAction);
		searchMenu.add(findNextAction);

		// View
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic('v');
		JMenuItem scrollToBottomMenuItem = new JMenuItem(scrollToBottomMenuAction);
		viewMenu.add(scrollToBottomMenuItem);
		viewMenu.add(pauseMenuAction);
		viewMenu.add(clearMenuAction);
		viewMenu.add(attachMenuAction);
		viewMenu.add(disconnectMenuAction);
		viewMenu.add(focusEventsAction);
		viewMenu.add(focusMessageAction);
		viewMenu.add(statisticsMenuAction);
		viewMenu.add(editSourceNameMenuAction);
		viewMenu.add(editConditionMenuAction);
		viewMenu.addSeparator();
		viewMenu.add(previousTabAction);
		viewMenu.add(nextTabAction);
		viewMenu.addSeparator();
		viewMenu.add(closeFilterAction);
		viewMenu.add(closeOtherFiltersAction);
		viewMenu.add(closeAllFiltersAction);

		// Window
		windowMenu = new JMenu("Window");
		windowMenu.setMnemonic('w');

		// Help
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('h');

		helpMenu.add(new JMenuItem(keyboardHelpAction));
		helpMenu.add(new JMenuItem(checkForUpdateAction));
		helpMenu.addSeparator();
		helpMenu.add(new JMenuItem(debugAction));
		if(!app.isMac())
		{
			helpMenu.addSeparator();
			helpMenu.add(new JMenuItem(aboutAction));
		}


		menubar.add(fileMenu);
		menubar.add(editMenu);
		menubar.add(searchMenu);
		menubar.add(viewMenu);
		menubar.add(windowMenu);
		menubar.add(helpMenu);

		updateWindowMenu();
		updateActions();
	}

	public PreferencesMenuAction getPreferencesAction()
	{
		return preferencesMenuAction;
	}

	public JToolBar getToolbar()
	{
		return toolbar;
	}

	public JMenuBar getMenuBar()
	{
		return menubar;
	}

	public void setViewContainer(ViewContainer viewContainer)
	{
		if(this.viewContainer!=viewContainer)
		{
			if(this.viewContainer!=null)
			{
				this.viewContainer.removeChangeListener(containerChangeListener);
				this.viewContainer.removePropertyChangeListener(containerPropertyChangeListener);
			}
			this.viewContainer = viewContainer;
			if(this.viewContainer!=null)
			{
				this.viewContainer.addChangeListener(containerChangeListener);
				this.viewContainer.addPropertyChangeListener(containerPropertyChangeListener);

				setEventWrapper(this.viewContainer.getSelectedEvent());
			}
			updateActions();
		}
	}

	public ViewContainer getViewContainer()
	{
		return viewContainer;
	}

	public void updateWindowMenu()
	{
		updateWindowMenu(windowMenu);
	}

	public void updateActions()
	{
		if(logger.isDebugEnabled()) //noinspection ThrowableInstanceNeverThrown
			logger.debug("updateActions()", new Throwable());
		boolean hasView=false;
		boolean hasFilter=false;
		boolean isActive=false;
		EventSource eventSource=null;
		if(viewContainer!=null)
		{
			hasView=true;
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventSource=eventWrapperViewPanel.getEventSource();
				hasFilter=eventWrapperViewPanel.getFilterCondition()!=null;
				isActive=eventWrapperViewPanel.getState() == LoggingViewState.ACTIVE;
			}
		}
		// Search
		searchMenu.setEnabled(hasView);
		findMenuAction.setEnabled(hasView);
		resetFindAction.setEnabled(hasFilter);
		findPreviousAction.setEnabled(hasFilter);
		findNextAction.setEnabled(hasFilter);

		// View
		viewMenu.setEnabled(hasView);
		scrollToBottomMenuAction.setEnabled(hasView);
		editSourceNameMenuAction.setEnabled(hasView);
		editConditionMenuAction.setEnabled(hasView);
		pauseMenuAction.setEnabled(hasView);
		clearMenuAction.setEnabled(hasView);
		attachMenuAction.setEnabled(hasView);
		disconnectMenuAction.setEnabled(isActive);
		focusEventsAction.setEnabled(hasView);
		focusMessageAction.setEnabled(hasView);
		statisticsMenuAction.setEnabled(hasView);
		nextTabAction.updateAction();
		previousTabAction.updateAction();
		closeFilterAction.setEnabled(hasView);
		closeOtherFiltersAction.setEnabled(hasView);
		closeAllFiltersAction.setEnabled(hasView);

		disconnectToolBarAction.setEnabled(isActive);

		scrollToBottomMenuAction.updateAction();
		editSourceNameMenuAction.updateAction();
		editConditionMenuAction.updateAction();
		pauseMenuAction.updateAction();
		attachMenuAction.updateAction();
		closeFilterAction.updateAction();
		closeOtherFiltersAction.updateAction();
		closeAllFiltersAction.updateAction();

		scrollToBottomButton.setSelected(isScrollingToBottom());
		pauseToolBarAction.updateAction();
		attachToolBarAction.updateAction();

		scrollToBottomToolBarAction.setEnabled(hasView);
		pauseToolBarAction.setEnabled(hasView);
		clearToolBarAction.setEnabled(hasView);
		findToolBarAction.setEnabled(hasView);
		statisticsToolBarAction.setEnabled(hasView);
		attachToolBarAction.setEnabled(hasView);
		disconnectToolBarAction.setEnabled(isActive);

		if(eventSource!=null)
		{
			showUnfilteredEventAction.setEnabled((eventSource.getFilter()!=null));
		}
		else
		{
			showUnfilteredEventAction.setEnabled(false);
		}
	}

	void setShowingFilters(boolean showingFilters)
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				eventWrapperViewPanel.setShowingFilters(showingFilters);
			}
		}
	}

	boolean isScrollingToBottom()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				return eventWrapperViewPanel.isScrollingToBottom();
			}
		}
		return false;
	}

	void setScrollingToBottom(boolean scrollingToBottom)
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				eventWrapperViewPanel.setScrollingToBottom(scrollingToBottom);
			}
		}
	}


	boolean isPaused()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				return eventWrapperViewPanel.isPaused();
			}
		}
		return false;
	}

	void setPaused(boolean paused)
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				eventWrapperViewPanel.setPaused(paused);
			}
		}
	}

	void clear()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				eventWrapperViewPanel.clear();
			}
		}
	}

	void focusTable()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.focusTable();
			}
		}
	}

	private void editCondition()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				Condition condition=eventWrapperViewPanel.getFilterCondition();
				if(condition!=null)
				{
					mainFrame.getPreferencesDialog().editCondition(condition);
				}
			}
		}
	}

	private void editSourceName()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel != null)
			{
				String sourceIdentifier=eventWrapperViewPanel.getEventSource().getSourceIdentifier().getIdentifier();
				if(!"global".equals(sourceIdentifier))
				{
					mainFrame.getPreferencesDialog().editSourceName(sourceIdentifier);
				}
			}
		}
	}

	private void attachDetach()
	{
		ViewContainer container = getViewContainer();
		if(container!=null)
		{
			MainFrame mainFrame=container.getMainFrame();
			ViewWindow window = container.resolveViewWindow();

			if(window instanceof JFrame)
			{
				window.closeWindow();
				mainFrame.showInternalFrame(container);
			}
			else if(window instanceof JInternalFrame)
			{
				window.closeWindow();
				mainFrame.showFrame(container);
			}
		}
		if(isScrollingToBottom())
		{
			if(viewContainer!=null)
			{
				EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
				if(eventWrapperViewPanel!=null)
				{
					eventWrapperViewPanel.scrollToBottom();
				}
			}
		}
		focusTable();
	}


	private void showStatistics()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				MainFrame mainFrame=viewContainer.getMainFrame();
				mainFrame.showStatistics(eventWrapperViewPanel.getEventSource().getSourceIdentifier());
			}
		}
	}

	private void disconnect()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.closeConnection(eventWrapperViewPanel.getEventSource().getSourceIdentifier());
			}
		}
	}

	private void focusMessage()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.focusMessagePane();
			}
		}
	}

	private void focusEvents()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.focusTable();
			}
		}
	}

	private void findNext()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.findNext(eventWrapperViewPanel.getSelectedRow(), eventWrapperViewPanel.getFilterCondition());
			}
		}
	}


	private void findPrevious()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.findPrevious(eventWrapperViewPanel.getSelectedRow(), eventWrapperViewPanel.getFilterCondition());
			}
		}
	}


	private void resetFind()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.resetFind();
			}
		}
	}

	private void closeCurrentFilter()
	{
		if(viewContainer!=null)
		{
			viewContainer.closeCurrentFilter();
		}
	}

	private void closeOtherFilters()
	{
		if(viewContainer!=null)
		{
			viewContainer.closeOtherFilters();
		}
	}

	private void closeAllFilters()
	{
		if(viewContainer!=null)
		{
			viewContainer.closeAllFilters();
		}
	}


	private void previousTab()
	{
		if(logger.isDebugEnabled()) logger.debug("PreviousTab");
		if(viewContainer!=null)
		{
			int viewCount=viewContainer.getViewCount();
			int viewIndex=viewContainer.getViewIndex();
			if(viewIndex>-1)
			{
				int newView = viewIndex - 1;
				if(newView <0)
				{
					newView =viewCount-1;
				}
				if(newView >=0 && newView <viewCount)
				{
					viewContainer.setViewIndex(newView);
				}
			}
		}
	}

	private void nextTab()
	{
		if(logger.isDebugEnabled()) logger.debug("NextTab");
		if(viewContainer!=null)
		{
			int viewIndex=viewContainer.getViewIndex();
			int viewCount=viewContainer.getViewCount();
			if(viewIndex>-1)
			{
				int newView = viewIndex + 1;
				if(newView >=viewCount)
				{
					newView =0;
				}
				if(newView >=0)
				{
					viewContainer.setViewIndex(newView);
				}
			}
		}
	}

	private void showUnfilteredEvent()
	{
		if(viewContainer!=null)
		{
			EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
			if(eventWrapperViewPanel!=null)
			{
				eventWrapperViewPanel.showUnfilteredEvent();
			}
		}
	}

	private void initPopup()
	{
		popup=new JPopupMenu();
		JMenuItem showUnfilteredMenuItem = new JMenuItem(showUnfilteredEventAction);
		Font f=showUnfilteredMenuItem.getFont();
		Font boldFont=f.deriveFont(Font.BOLD);
		showUnfilteredMenuItem.setFont(boldFont);

		popup.add(showUnfilteredMenuItem);

		JMenu copyMenuItem = new JMenu("Copy...");
		popup.add(copyMenuItem);
		copyMenuItem.add(new JMenuItem(copyEventAction));
		copyMenuItem.addSeparator();
		copyMenuItem.add(new JMenuItem(copyLoggingMessageAction));
		copyMenuItem.add(new JMenuItem(copyLoggerNameAction));
		copyMenuItem.add(new JMenuItem(copyLoggingThrowableAction));
		copyMenuItem.addSeparator();
		copyMenuItem.add(new JMenuItem(copyAccessUriAction));

		sendToMenuItem = new JMenu("Send to...");

		popup.add(sendToMenuItem);

		JMenuItem gotoSourceMenuItem = new JMenuItem(gotoSourceAction);
		popup.add(gotoSourceMenuItem);
	}

	private void setEventWrapper(EventWrapper wrapper)
	{
		this.eventWrapper=wrapper;
		gotoSourceAction.setEventWrapper(wrapper);
		copyEventAction.setEventWrapper(wrapper);
		copyLoggingMessageAction.setEventWrapper(wrapper);
		copyLoggerNameAction.setEventWrapper(wrapper);
		copyLoggingThrowableAction.setEventWrapper(wrapper);
		copyAccessUriAction.setEventWrapper(wrapper);
	}

	public void updateWindowMenu(JMenu windowMenu)
	{
		// must be executed later because the ancestor-change-event is fired
		// while parent is still != null...
		// see JComponent.removeNotify source for comment.
		SwingUtilities.invokeLater(new UpdateWindowMenuRunnable(windowMenu));
	}

	public ActionListener getAboutAction()
	{
		return aboutAction;
	}

	public void updatePopup()
	{
		if(popup==null)
		{
			initPopup();
		}
		sendToMenuItem.removeAll();
		if(eventWrapper==null)
		{
			sendToMenuItem.setEnabled(false);
		}
		else
		{
			Serializable eventObj = eventWrapper.getEvent();
			if(eventObj instanceof LoggingEvent)
			{
				Map<String, EventSender<LoggingEvent>> senders = mainFrame.getLoggingEventSenders();
				if(logger.isInfoEnabled()) logger.info("Senders: {}", senders);
				if(senders.size()==0)
				{
					sendToMenuItem.setEnabled(false);
				}
				else
				{
					sendToMenuItem.setEnabled(true);
					for(Map.Entry<String, EventSender<LoggingEvent>> current: senders.entrySet())
					{
						SendAction<LoggingEvent> action = new SendAction<LoggingEvent>(current.getKey(), current.getValue(), eventWrapper);
						JMenuItem menuItem=new JMenuItem(action);
						sendToMenuItem.add(menuItem);
					}
				}
			}
			else if(eventObj instanceof AccessEvent)
			{
				Map<String, EventSender<AccessEvent>> senders = mainFrame.getAccessEventSenders();
				if(logger.isInfoEnabled()) logger.info("Senders: {}", senders);
				if(senders.size()==0)
				{
					sendToMenuItem.setEnabled(false);
				}
				else
				{
					sendToMenuItem.setEnabled(true);
					for(Map.Entry<String, EventSender<AccessEvent>> current: senders.entrySet())
					{
						SendAction<AccessEvent> action = new SendAction<AccessEvent>(current.getKey(), current.getValue(), eventWrapper);
						JMenuItem menuItem=new JMenuItem(action);
						sendToMenuItem.add(menuItem);
					}
				}
			}
			else
			{
				sendToMenuItem.setEnabled(false);
			}
		}
	}

	public JPopupMenu getPopupMenu()
	{
		updatePopup();

		return popup;
	}

/*
	private class ClearAndRemoveInactiveAction
		extends AbstractAction
	{

		public ClearAndRemoveInactiveAction()
		{
			super("Clean and remove inactive");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift R");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('c'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.removeInactiveViews(false, true);
			mainFrame.updateWindowMenus();
		}
	}
*/
	
	private class RemoveInactiveAction
		extends AbstractAction
	{

		public RemoveInactiveAction()
		{
			super("Remove inactive");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" R");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.removeInactiveViews(false, false);
			mainFrame.updateWindowMenus();
		}
	}

	private class CloseAllAction
		extends AbstractAction
	{

		public CloseAllAction()
		{
			super("Close all");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			//KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" R");
			//if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			//putValue(Action.ACCELERATOR_KEY, accelerator);
			//putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.closeAllViews(null);
			mainFrame.updateWindowMenus();
		}
	}

	private class CloseAllOtherAction
		extends AbstractAction
	{

		public CloseAllOtherAction()
		{
			super("Close all other");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			//KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" R");
			//if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			//putValue(Action.ACCELERATOR_KEY, accelerator);
			//putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.closeAllViews(viewContainer);
			mainFrame.updateWindowMenus();
		}
	}

	private class MinimizeAllAction
		extends AbstractAction
	{

		public MinimizeAllAction()
		{
			super("Minimize all");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			//KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" R");
			//if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			//putValue(Action.ACCELERATOR_KEY, accelerator);
			//putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.minimizeAllViews(null);
			mainFrame.updateWindowMenus();
		}
	}

	private class MinimizeAllOtherAction
		extends AbstractAction
	{

		public MinimizeAllOtherAction()
		{
			super("Minimize all other");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			//KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" R");
			//if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			//putValue(Action.ACCELERATOR_KEY, accelerator);
			//putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.minimizeAllViews(viewContainer);
			mainFrame.updateWindowMenus();
		}
	}
	
	private class ClearToolBarAction
		extends AbstractAction
	{

		public ClearToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, CLEAR_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Clear");
		}

		public void actionPerformed(ActionEvent e)
		{
			clear();
		}
	}

	private class ClearMenuAction
		extends AbstractAction
	{
		public ClearMenuAction()
		{
			super("Clear");
			putValue(Action.SMALL_ICON, CLEAR_MENU_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Clear this view.");
		}

		public void actionPerformed(ActionEvent e)
		{
			clear();
		}
	}

	private class EditConditionMenuAction
		extends AbstractAction
	{
		public EditConditionMenuAction()
		{
			super("Add condition...");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" I");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Add the condition of the current view.");
		}

		public void actionPerformed(ActionEvent e)
		{
			editCondition();
		}

		public void updateAction()
		{
			boolean enable=false;
			if(viewContainer!=null)
			{
				EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
				if(eventWrapperViewPanel != null)
				{
					Condition condition=eventWrapperViewPanel.getFilterCondition();
					if(condition!=null)
					{
					    enable=true;
					}
				}
			}
			setEnabled(enable);
		}
	}

	private class EditSourceNameMenuAction
		extends AbstractAction
	{
		public EditSourceNameMenuAction()
		{
			super("Edit source name...");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" B");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Edit the source name of the current view.");
		}

		public void actionPerformed(ActionEvent e)
		{
			editSourceName();
		}


		public void updateAction()
		{
			boolean enable=false;
			if(viewContainer!=null)
			{
				EventWrapperViewPanel eventWrapperViewPanel = viewContainer.getSelectedView();
				if(eventWrapperViewPanel != null)
				{
					String sourceIdentifier=eventWrapperViewPanel.getEventSource().getSourceIdentifier().getIdentifier();
					if(!"global".equals(sourceIdentifier))
					{
					    enable=true;
					}
				}
			}
			setEnabled(enable);
		}
	}


	private class AttachMenuAction
		extends AbstractAction
	{
		private Icon attachIcon=ATTACH_MENU_ICON;
		private Icon detachIcon=DETACH_MENU_ICON;

		public AttachMenuAction()
		{
			super();
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift A");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			updateAction();
		}

		public void actionPerformed(ActionEvent e)
		{
			attachDetach();
			updateAction();
		}

		public void updateAction()
		{
			ViewContainer container = getViewContainer();
			if(container!=null)
			{
				ViewWindow window = container.resolveViewWindow();
				if(window instanceof JInternalFrame)
				{
					putValue(Action.SMALL_ICON, detachIcon);
					putValue(Action.NAME, "Detach");
				}
				else if(window instanceof JFrame)
				{
					putValue(Action.SMALL_ICON, attachIcon);
					putValue(Action.NAME, "Attach");
				}
			}

		}
	}


	private class AttachToolBarAction
		extends AbstractAction
	{
		private Icon attachIcon=ATTACH_TOOLBAR_ICON;
		private Icon detachIcon=DETACH_TOOLBAR_ICON;

		public AttachToolBarAction()
		{
			super();
			updateAction();
		}

		public void actionPerformed(ActionEvent e)
		{
			attachDetach();
			updateAction();
		}

		public void updateAction()
		{
			ViewContainer container = getViewContainer();
			if(container!=null)
			{
				ViewWindow window = container.resolveViewWindow();
				if(window instanceof JInternalFrame)
				{
					putValue(Action.SMALL_ICON, detachIcon);
					putValue(Action.SHORT_DESCRIPTION, "Detach");
					return;
				}
				else if(window instanceof JFrame)
				{
					putValue(Action.SMALL_ICON, attachIcon);
					putValue(Action.SHORT_DESCRIPTION, "Attach");
					return;
				}
			}
			// update anyway
			putValue(Action.SMALL_ICON, detachIcon);
			putValue(Action.SHORT_DESCRIPTION, "Detach");
		}
	}

	private class PauseMenuAction
		extends AbstractAction
	{
		private Icon pausedIcon=PAUSED_MENU_ICON;
		private Icon unpausedIcon=UNPAUSED_MENU_ICON;

		public PauseMenuAction()
		{
			super();
			updateAction();
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" P");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			setPaused(!isPaused());
			updateAction();
			focusTable();
		}

		public void updateAction()
		{
			if(isPaused())
			{
				putValue(Action.SMALL_ICON, pausedIcon);
				putValue(Action.NAME, "Unpause");
			}
			else
			{
				putValue(Action.SMALL_ICON, unpausedIcon);
				putValue(Action.NAME, "Pause");
			}
		}
	}

	private class PauseToolBarAction
		extends AbstractAction
	{
		private Icon pausedIcon=PAUSED_TOOLBAR_ICON;
		private Icon unpausedIcon=UNPAUSED_TOOLBAR_ICON;

		public PauseToolBarAction()
		{
			super();
			updateAction();
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" P");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			setPaused(!isPaused());
			updateAction();
			focusTable();
		}

		public void updateAction()
		{
			if(isPaused())
			{
				putValue(Action.SMALL_ICON, pausedIcon);
				putValue(Action.SHORT_DESCRIPTION, "Unpause");
			}
			else
			{
				putValue(Action.SMALL_ICON, unpausedIcon);
				putValue(Action.SHORT_DESCRIPTION, "Pause");
			}
		}
	}

	private class FindMenuAction
		extends AbstractAction
	{
		public FindMenuAction()
		{
			super("Find");
			putValue(Action.SMALL_ICON, FIND_MENU_ITEM);
			putValue(Action.SHORT_DESCRIPTION, "Opens the Find panel.");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" F");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			setShowingFilters(true);
		}
	}

	private class FindToolBarAction
		extends AbstractAction
	{
		public FindToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, FIND_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Find");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" F");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			setShowingFilters(true);
		}
	}

	private class StatisticsMenuAction
		extends AbstractAction
	{
		public StatisticsMenuAction()
		{
			super("Statistics");
			putValue(Action.SMALL_ICON, STATISTICS_MENU_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Statistics");
		}

		public void actionPerformed(ActionEvent e)
		{
			showStatistics();
		}
	}

	private class StatisticsToolBarAction
		extends AbstractAction
	{
		public StatisticsToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, STATISTICS_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Statistics");
		}

		public void actionPerformed(ActionEvent e)
		{
			showStatistics();
		}
	}

	private class DisconnectMenuAction
		extends AbstractAction
	{
		public DisconnectMenuAction()
		{
			super("Disconnect");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift D");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.SMALL_ICON, DISCONNECT_MENU_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Terminates this connection");
		}

		public void actionPerformed(ActionEvent e)
		{
			disconnect();
		}
	}

	private class DisconnectToolBarAction
		extends AbstractAction
	{
		public DisconnectToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, DISCONNECT_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Disconnect");
		}

		public void actionPerformed(ActionEvent e)
		{
			disconnect();
		}
	}

	private class FocusMessageAction
		extends AbstractAction
	{
		public FocusMessageAction()
		{
			super("Focus message");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Focus detailed message view.");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" M");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			focusMessage();
		}
	}

	private class FocusEventsAction
		extends AbstractAction
	{
		public FocusEventsAction()
		{
			super("Focus events");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Focus the table containing the events.");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" E");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			focusEvents();
		}
	}

	private class FindNextAction
		extends AbstractAction
	{
		public FindNextAction()
		{
			super("Find next");
			putValue(Action.SMALL_ICON, FIND_NEXT_MENU_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Find next match of the current filter.");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift G");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			findNext();
		}

	}

	private class FindPreviousAction
		extends AbstractAction
	{
		public FindPreviousAction()
		{
			super("Find previous");
			putValue(Action.SMALL_ICON, FIND_PREV_MENU_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Find previous match of the current filter.");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" G");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			findPrevious();
		}
	}

	private class ResetFindAction
		extends AbstractAction
	{
		public ResetFindAction()
		{
			super("Reset find");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift F");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			resetFind();
		}
	}

	private class ScrollToBottomMenuAction
		extends AbstractAction
	{
		private Icon selectedIcon=TAIL_MENU_ICON;
		private Icon unselectedIcon=EMPTY_16_ICON;

		public ScrollToBottomMenuAction()
		{
			super("Tail");
			updateAction();
			putValue(Action.SHORT_DESCRIPTION, "Tail (\"scroll to bottom\")");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" T");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			boolean tail=!isScrollingToBottom();
			setScrollingToBottom(tail);
			if(logger.isDebugEnabled()) logger.debug("tail={}",tail);
			focusTable();
		}

		public void updateAction()
		{
			if(isScrollingToBottom())
			{
				putValue(Action.SMALL_ICON, selectedIcon);
			}
			else
			{
				putValue(Action.SMALL_ICON, unselectedIcon);
			}
		}
	}

	private class ScrollToBottomToolBarAction
		extends AbstractAction
	{

		public ScrollToBottomToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, TAIL_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Tail (\"scroll to bottom\")");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" T");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void actionPerformed(ActionEvent e)
		{
			boolean tail=!isScrollingToBottom();
			setScrollingToBottom(tail);
			if(logger.isDebugEnabled()) logger.debug("tail={}",tail);
			focusTable();
		}
	}

	private class CloseFilterAction
		extends AbstractAction
	{
		public CloseFilterAction()
		{
			super("Close this filter");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('c'));
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" W");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void updateAction()
		{
			if(viewContainer!=null)
			{
				int viewIndex=viewContainer.getViewIndex();
				if(viewIndex>0)
				{
					setEnabled(true);
				}
				else
				{
					setEnabled(false);
				}
			}
			else
			{
				setEnabled(false);
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			closeCurrentFilter();
		}

	}

	private class CloseOtherFiltersAction
		extends AbstractAction
	{
		public CloseOtherFiltersAction()
		{
			super("Close all other filters");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('o'));
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift W");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
		}

		public void updateAction()
		{
			if(viewContainer!=null)
			{
				int viewIndex=viewContainer.getViewIndex();
				int viewCount=viewContainer.getViewCount();
				if(viewIndex>-1 && ((viewIndex==0 && viewCount>1) || viewCount>2))
				{
					setEnabled(true);
				}
				else
				{
					setEnabled(false);
				}
			}
			else
			{
				setEnabled(false);
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			closeOtherFilters();
		}

	}

	private class CloseAllFiltersAction
		extends AbstractAction
	{
		public CloseAllFiltersAction()
		{
			super("Close all filters");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('a'));
		}

		public void updateAction()
		{
			int viewCount=0;
			if(viewContainer!=null)
			{
				viewCount=viewContainer.getViewCount();
			}
			if(viewCount>1)
			{
				setEnabled(true);
			}
			else
			{
				setEnabled(false);
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			closeAllFilters();
		}
	}



	class ViewLoggingAction
		extends AbstractAction
	{
		private EventSource<LoggingEvent> eventSource;

		public ViewLoggingAction(EventSource<LoggingEvent> eventSource)
		{
			super(mainFrame.getLoggingSourceTitle(eventSource.getSourceIdentifier()));
			this.eventSource=eventSource;
			if(eventSource.isGlobal())
			{
				KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" 1");
				if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
				putValue(Action.ACCELERATOR_KEY, accelerator);
			}
		}

		public void actionPerformed(ActionEvent evt)
		{
			mainFrame.showLoggingView(eventSource);
		}

	}

	class ViewAccessAction
		extends AbstractAction
	{
		private EventSource<AccessEvent> eventSource;

		public ViewAccessAction(EventSource<AccessEvent> eventSource)
		{
			super(mainFrame.getAccessSourceTitle(eventSource.getSourceIdentifier()));
			this.eventSource=eventSource;
			if(eventSource.isGlobal())
			{
				KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" 2");
				if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
				putValue(Action.ACCELERATOR_KEY, accelerator);
			}
		}

		public void actionPerformed(ActionEvent evt)
		{
			mainFrame.showAccessView(eventSource);
		}

	}

	class ViewStatisticsAction
		extends AbstractAction
	{
		private SourceIdentifier sourceIentifier;

		public ViewStatisticsAction(String name, SourceIdentifier sourceIdentifier)
		{
			super(name);
			this.sourceIentifier=sourceIdentifier;
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showStatistics(sourceIentifier);
		}
	}

	static class StatisticsSubMenuAction
		extends AbstractAction
	{

		public StatisticsSubMenuAction()
		{
			super("Statistics");
			putValue(Action.SMALL_ICON, STATISTICS_MENU_ICON);
		}

		public void actionPerformed(ActionEvent e)
		{
		}
	}


	class UpdateWindowMenuRunnable implements Runnable
	{
		private JMenu windowMenu;

		public UpdateWindowMenuRunnable(JMenu windowMenu)
		{
			this.windowMenu = windowMenu;
		}

		public void run()
		{
			// remove loggingViews that were closed in the meantime...
			mainFrame.removeInactiveViews(true, false);
			if(logger.isDebugEnabled()) logger.debug("Updating Views-Menu.");

			windowMenu.removeAll();
			JMenu statisticsMenu=createStatisticsMenu();
			windowMenu.add(statisticsMenu);
			windowMenu.add(closeAllItem);
			windowMenu.add(closeAllOtherItem);
			windowMenu.add(minimizeAllItem);
			windowMenu.add(minimizeAllOtherItem);
			windowMenu.add(removeInactiveItem);
//			windowMenu.add(clearAndRemoveInactiveItem);

			int activeCounter=0;
			int inactiveCounter=0;
			int viewCounter=0;
			Font inactiveFont = windowMenu.getFont();
			inactiveFont =inactiveFont.deriveFont(Font.PLAIN);

			boolean first;

			SortedMap<EventSource<LoggingEvent>, ViewContainer<LoggingEvent>> sortedLoggingViews=mainFrame.getSortedLoggingViews();

			SortedMap<EventSource<AccessEvent>, ViewContainer<AccessEvent>> sortedAccessViews=mainFrame.getSortedAccessViews();

			// global (Logging & Access)
			first=true;
			for (Map.Entry<EventSource<LoggingEvent>, ViewContainer<LoggingEvent>> entry : sortedLoggingViews.entrySet())
			{
				EventSource<LoggingEvent> key = entry.getKey();
				ViewContainer<LoggingEvent> value = entry.getValue();
				if(value.resolveViewWindow()!=null)
				{
					viewCounter++;
				}
				if (key.isGlobal())
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createLoggingMenuItem(key, value);
					windowMenu.add(menuItem);
				}
			}
			for (Map.Entry<EventSource<AccessEvent>, ViewContainer<AccessEvent>> entry : sortedAccessViews.entrySet())
			{
				EventSource<AccessEvent> key = entry.getKey();
				ViewContainer<AccessEvent> value = entry.getValue();
				if(value.resolveViewWindow()!=null)
				{
					viewCounter++;
				}
				if (key.isGlobal())
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createAccessMenuItem(key, value);
					windowMenu.add(menuItem);
				}
			}

			first=true;
			// Logging (active)
			for (Map.Entry<EventSource<LoggingEvent>, ViewContainer<LoggingEvent>> entry : sortedLoggingViews.entrySet())
			{
				EventSource<LoggingEvent> key = entry.getKey();
				ViewContainer<LoggingEvent> value = entry.getValue();
				EventWrapperViewPanel<LoggingEvent> panel = value.getDefaultView();
				if(!key.isGlobal() && (LoggingViewState.ACTIVE == panel.getState()))
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createLoggingMenuItem(key, value);
					windowMenu.add(menuItem);
					activeCounter++;
				}
			}
			// Logging (inactive)
			for (Map.Entry<EventSource<LoggingEvent>, ViewContainer<LoggingEvent>> entry : sortedLoggingViews.entrySet())
			{
				EventSource<LoggingEvent> key = entry.getKey();
				ViewContainer<LoggingEvent> value = entry.getValue();
				EventWrapperViewPanel<LoggingEvent> panel = value.getDefaultView();
				if(!key.isGlobal() && (LoggingViewState.ACTIVE != panel.getState()))
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createLoggingMenuItem(key, value);
					menuItem.setFont(inactiveFont);
					windowMenu.add(menuItem);
					inactiveCounter++;
				}
			}

			// Access (active)
			first=true;
			for (Map.Entry<EventSource<AccessEvent>, ViewContainer<AccessEvent>> entry : sortedAccessViews.entrySet())
			{
				EventSource<AccessEvent> key = entry.getKey();
				ViewContainer<AccessEvent> value = entry.getValue();
				EventWrapperViewPanel<AccessEvent> panel = value.getDefaultView();
				if(!key.isGlobal() && (LoggingViewState.ACTIVE == panel.getState()))
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createAccessMenuItem(key, value);
					windowMenu.add(menuItem);
					activeCounter++;
				}
			}
			// Access (inactive)
			for (Map.Entry<EventSource<AccessEvent>, ViewContainer<AccessEvent>> entry : sortedAccessViews.entrySet())
			{
				EventSource<AccessEvent> key = entry.getKey();
				ViewContainer<AccessEvent> value = entry.getValue();
				EventWrapperViewPanel<AccessEvent> panel = value.getDefaultView();
				if(!key.isGlobal() && (LoggingViewState.ACTIVE != panel.getState()))
				{
					if (first)
					{
						first = false;
						windowMenu.addSeparator();
					}
					JCheckBoxMenuItem menuItem = createAccessMenuItem(key, value);
					menuItem.setFont(inactiveFont);
					windowMenu.add(menuItem);
					inactiveCounter++;
				}
			}

			// update status text
			boolean hasInactive=(inactiveCounter!=0);
			//clearAndRemoveInactiveAction.setEnabled(hasInactive);
			removeInactiveAction.setEnabled(hasInactive);
			boolean hasViews=viewCounter!=0;
			minimizeAllAction.setEnabled(hasViews);
			closeAllAction.setEnabled(hasViews);
			if(viewContainer==null || viewCounter<=1)
			{
				minimizeAllOtherAction.setEnabled(false);
				closeAllOtherAction.setEnabled(false);
			}
			else
			{
				minimizeAllOtherAction.setEnabled(true);
				closeAllOtherAction.setEnabled(true);
			}

			mainFrame.setActiveConnectionsCounter(activeCounter);

			if(windowMenu.isPopupMenuVisible())
			{
				// I've not been able to find a more elegant solution to prevent
				// repaint artifacts if the menu contents change while the menu is still open...
				windowMenu.setPopupMenuVisible(false);
				windowMenu.setPopupMenuVisible(true);
			}
		}


		private JCheckBoxMenuItem createLoggingMenuItem(EventSource<LoggingEvent> key, ViewContainer<LoggingEvent> viewContainer)
		{
			JCheckBoxMenuItem result = new JCheckBoxMenuItem(new ViewLoggingAction(key));
			Container compParent = viewContainer.getParent();
			if(logger.isDebugEnabled()) logger.debug("\n\nParent for {}: {}\n", key.getSourceIdentifier(), compParent);
			if (compParent == null)
			{
				result.setState(false);
			}
			else
			{
				result.setState(true);
			}
			return result;
		}

		private JCheckBoxMenuItem createAccessMenuItem(EventSource<AccessEvent> key, ViewContainer<AccessEvent> viewContainer)
		{
			JCheckBoxMenuItem result = new JCheckBoxMenuItem(new ViewAccessAction(key));
			Container compParent = viewContainer.getParent();
			if(logger.isDebugEnabled()) logger.debug("\n\nParent for {}: {}\n", key.getSourceIdentifier(), compParent);
			if (compParent == null)
			{
				result.setState(false);
			}
			else
			{
				result.setState(true);
			}
			return result;
		}

		private JMenu createStatisticsMenu()
		{
			JMenu result = new JMenu(new StatisticsMenuAction());
			SortedMap<String, SourceIdentifier> sources=mainFrame.getAvailableStatistics();

			{
				JMenuItem menuItem=new JMenuItem(new ViewStatisticsAction("Global", new SourceIdentifier("global")));
				result.add(menuItem);
				result.addSeparator();
			}

			for(Map.Entry<String,SourceIdentifier> current:sources.entrySet())
			{
				String key=current.getKey();
				SourceIdentifier value = current.getValue();

				JMenuItem menuItem=new JMenuItem(new ViewStatisticsAction(key, value));
				result.add(menuItem);
			}
			return result;
		}
	}


	class AboutAction
		extends AbstractAction
	{

		public AboutAction()
		{
			super("About...");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showAboutDialog();
		}
	}

	class CheckForUpdateAction
		extends AbstractAction
	{

		public CheckForUpdateAction()
		{
			super("Check for Update...");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.checkForUpdate(true);
		}
	}

	class KeyboardHelpAction
		extends AbstractAction
	{
		public KeyboardHelpAction()
		{
			super("Help Topics");
			putValue(Action.SMALL_ICON, HELP_MENU_ICON);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showHelp();
		}
	}

	class PreferencesMenuAction
		extends AbstractAction
	{

		public PreferencesMenuAction()
		{
			super("Preferences...");
			putValue(Action.SMALL_ICON, PREFERENCES_MENU_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" COMMA");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('p'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showPreferencesDialog();
		}
	}

	class PreferencesToolBarAction
		extends AbstractAction
	{

		public PreferencesToolBarAction()
		{
			super();
			putValue(Action.SMALL_ICON, PREFERENCES_TOOLBAR_ICON);
			putValue(Action.SHORT_DESCRIPTION, "Preferences...");
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" COMMA");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('p'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showPreferencesDialog();
		}
	}

	class DebugAction
		extends AbstractAction
	{

		public DebugAction()
		{
			super("Debug");
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.showDebugDialog();
		}
	}

	class ExitMenuAction
		extends AbstractAction
	{

		public ExitMenuAction()
		{
			super("Exit");
			putValue(Action.SMALL_ICON, EXIT_MENU_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" Q");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('x'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.exit();
		}
	}

	class OpenInactiveLogMenuAction
		extends AbstractAction
	{
		public OpenInactiveLogMenuAction()
		{
			super("Open inactive log...");
			putValue(Action.SMALL_ICON, OPEN_INACTIVE_MENU_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift O");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('o'));
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.openInactiveLogs();
		}
	}

	class CleanAllInactiveLogsMenuAction
		extends AbstractAction
	{

		public CleanAllInactiveLogsMenuAction()
		{
			super("Clean all inactive logs");
			putValue(Action.SMALL_ICON, CLEAR_MENU_ICON);
			KeyStroke accelerator= KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" shift X");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.MNEMONIC_KEY, Integer.valueOf('c'));
		}

		public void actionPerformed(ActionEvent e)
		{
			if(logger.isInfoEnabled()) logger.info("Clean all inactive logs");
			mainFrame.cleanAllInactiveLogs();
		}
	}

	private class PreviousTabAction
		extends AbstractAction
	{
		public PreviousTabAction()
		{
			super("Previous tab");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" J");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
		}

		public void updateAction()
		{
			if(viewContainer!=null)
			{
				int viewCount=viewContainer.getViewCount();
				if(viewCount>1)
				{
					setEnabled(true);
				}
				else
				{
					setEnabled(false);
				}
			}
			else
			{
				setEnabled(false);
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			previousTab();
		}
	}

	private class NextTabAction
		extends AbstractAction
	{
		public NextTabAction()
		{
			super("Next tab");
			KeyStroke accelerator=KeyStrokes.resolveAcceleratorKeyStroke(KeyStrokes.COMMAND_ALIAS+" K");
			if(logger.isDebugEnabled()) logger.debug("accelerator: {}", accelerator);
			putValue(Action.ACCELERATOR_KEY, accelerator);
			putValue(Action.SMALL_ICON, EMPTY_16_ICON);
		}

		public void updateAction()
		{
			if(viewContainer!=null)
			{
				int viewCount=viewContainer.getViewCount();
				if(viewCount>1)
				{
					setEnabled(true);
				}
				else
				{
					setEnabled(false);
				}
			}
			else
			{
				setEnabled(false);
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			nextTab();
		}
	}


	private class CopyEventAction
		extends AbstractAction
	{
		private EventWrapper eventWrapper;

		public CopyEventAction()
		{
			super("Copy event");
			putValue(Action.SHORT_DESCRIPTION, "Copies the event to the clipboard.");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			this.eventWrapper=wrapper;
			if(wrapper==null)
			{
				setEnabled(false);
			}
			else
			{
				Serializable eventObj = wrapper.getEvent();
				if(eventObj instanceof LoggingEvent || eventObj instanceof AccessEvent)
				{
					setEnabled(true);
				}
				else
				{
					setEnabled(false);
				}
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			String message=mainFrame.createMessage(eventWrapper);
			mainFrame.copyHtml(message);
		}
	}

	private class CopyLoggingMessageAction
		extends AbstractAction
	{
		private String message;

		public CopyLoggingMessageAction()
		{
			super("Copy message");
			putValue(Action.SHORT_DESCRIPTION, "Copies the message of the logging event to the clipboard.");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			String message=null;
			if(wrapper!=null && wrapper.getEvent()!=null)
			{
				Object eventObj=wrapper.getEvent();
				if(eventObj instanceof LoggingEvent)
				{
					LoggingEvent loggingEvent = (LoggingEvent) eventObj;
					message=loggingEvent.getMessage();
				}
			}
			setMessage(message);
		}

		private void setMessage(String message)
		{
			this.message=message;
			setEnabled(message!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.copyText(message);
		}
	}

	private class CopyLoggerNameAction
		extends AbstractAction
	{
		private String loggerName;

		public CopyLoggerNameAction()
		{
			super("Copy logger name");
			putValue(Action.SHORT_DESCRIPTION, "Copies the logger name of the logging event to the clipboard.");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			String loggerName=null;
			if(wrapper!=null && wrapper.getEvent()!=null)
			{
				Object eventObj=wrapper.getEvent();
				if(eventObj instanceof LoggingEvent)
				{
					LoggingEvent loggingEvent = (LoggingEvent) eventObj;
					loggerName=loggingEvent.getLogger();
				}
			}
			setLoggerName(loggerName);
		}

		private void setLoggerName(String loggerName)
		{
			this.loggerName=loggerName;
			setEnabled(loggerName!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.copyText(loggerName);
		}
	}

	private class CopyLoggingThrowableAction
		extends AbstractAction
	{
		private ThrowableInfo throwableInfo;

		public CopyLoggingThrowableAction()
		{
			super("Copy Throwable");
			putValue(Action.SHORT_DESCRIPTION, "Copies the Throwable of the logging event to the clipboard.");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			ThrowableInfo info=null;
			if(wrapper!=null && wrapper.getEvent()!=null)
			{
				Object eventObj=wrapper.getEvent();
				if(eventObj instanceof LoggingEvent)
				{
					LoggingEvent loggingEvent = (LoggingEvent) eventObj;
					info=loggingEvent.getThrowable();
				}
			}
			setThrowableInfo(info);
		}

		private void setThrowableInfo(ThrowableInfo info)
		{
			this.throwableInfo=info;
			setEnabled(info!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			StringBuilder throwableText=new StringBuilder();
			ThrowableInfo info=throwableInfo;
			for(;;)
			{
				throwableText.append("Exception: ").append(info.getName()).append("\n");
				String message=info.getMessage();
				if(message!=null && !message.equals(info.getName()))
				{
					throwableText.append("Message: ").append(message).append("\n");
				}
				StackTraceElement[] st = info.getStackTrace();
				if(st!=null)
				{
					throwableText.append("StackTrace:\n");
					for(StackTraceElement current:st)
					{
						throwableText.append("\t").append(current).append("\n");
					}
				}
				info=info.getCause();
				if(info==null)
				{
					break;
				}
				throwableText.append("\nCaused by:\n");
			}
			mainFrame.copyText(throwableText.toString());
		}
	}

	private class CopyAccessUriAction
		extends AbstractAction
	{
		private String requestUri;

		public CopyAccessUriAction()
		{
			super("Copy request URI");
			putValue(Action.SHORT_DESCRIPTION, "Copies the request URI of the access event to the clipboard.");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			String requestUrl=null;
			if(wrapper!=null && wrapper.getEvent()!=null)
			{
				Object eventObj=wrapper.getEvent();
				if(eventObj instanceof AccessEvent)
				{
					AccessEvent accessEvent = (AccessEvent) eventObj;
					requestUrl=accessEvent.getRequestURI();
				}
			}
			setRequestUri(requestUrl);
		}

		private void setRequestUri(String requestUri)
		{
			this.requestUri = requestUri;
			setEnabled(requestUri !=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.copyText(requestUri);
		}
	}

	private class ShowUnfilteredEventAction
		extends AbstractAction
	{

		public ShowUnfilteredEventAction()
		{
			super("Show unfiltered");
			putValue(Action.SHORT_DESCRIPTION, "Show unfiltered event.");
		}

		public void actionPerformed(ActionEvent e)
		{
			showUnfilteredEvent();
		}

	}

	private class GotoSourceAction
		extends AbstractAction
	{
		private StackTraceElement stackTraceElement;

		public GotoSourceAction()
		{
			super("Go to source");
			putValue(Action.SHORT_DESCRIPTION, "Show source in IDEA if everything is ok ;)");
		}

		public void setEventWrapper(EventWrapper wrapper)
		{
			if(wrapper==null)
			{
				setStackTraceElement(null);
				return;
			}
			Serializable event=wrapper.getEvent();
			if(event instanceof LoggingEvent)
			{
				LoggingEvent loggingEvent = (LoggingEvent) event;
				StackTraceElement[] callStack = loggingEvent.getCallStack();
				if(callStack!=null && callStack.length>0)
				{
					setStackTraceElement(callStack[0]);
					return;
				}
			}
			setStackTraceElement(null);
		}

		public void setStackTraceElement(StackTraceElement stackTraceElement)
		{
			this.stackTraceElement=stackTraceElement;
			setEnabled(this.stackTraceElement!=null);
		}

		public void actionPerformed(ActionEvent e)
		{
			mainFrame.goToSource(stackTraceElement);
		}
	}

	protected class SendAction<T extends Serializable>
		extends AbstractAction
	{
		private EventSender<T> sender;
		private T event;

		public SendAction(String name, EventSender<T> sender, EventWrapper<T> wrapper)
		{
			super(name);
			this.sender = sender;
			this.event = wrapper.getEvent();
			setEnabled(event!=null);
		}

		/**
		 * Invoked when an action occurs.
		 */
		public void actionPerformed(ActionEvent e)
		{
			sender.send(event);
		}
	}
}
